import matplotlib
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, RadioButtons

import numpy as np
matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
import multiprocessing
import time
import random
import sys
from Tkinter import *
import spidev

# no number greater than 2^8 should be sent in here
def float2fix7(val):
    return int(val*128.0)

def fix2float7(val):
    return val/128.0

spi = spidev.SpiDev()
spi.open(0,0)
spi.mode = 0
spi.max_speed_hz = 20000000

data_width = 0.002
acc = 0
xFirst = 0
xIncrement = 0.005
curTime = 0
prevTime = 0
numSamples = 1200
saveIndex = 0

#Create a window
window=Tk()
xVal = []
yVal = []
i = data_width
packet = [0x00,float2fix7(0.5),0,0x32]
prevPacket = [0,0,0,0]
changedPacket = False
samplesToDropMultiple = 100
peakToPeak = 0

tValue = 0
x = 0

def main():
    #Create a queue to share data between process
    q1 = multiprocessing.Queue()
    q2 = multiprocessing.Queue()
    

    #Create and start the simulation process
    simulate=multiprocessing.Process(None,simulation,args=(q1,q2,))
    simulate.start()

    #Create the base plot
    plot()

    #Call a function to update the plot when there is new data
    updateplot(q1, q2)

    window.mainloop()
    print 'Done'

def updateAxes(x):
    global tValue
    global ymax
    global data_width

    tValue = int(xMaxSlider.val)
    #data_width = int(xmax)
    #ax.set_xlim(0, data_width)
    

    ymax = yMaxSlider.val
    ax.set_ylim(0, int(ymax))

# Type: 0 is Sine, 1 is Square, 2 is Sawtooth
def fcn_gen(x):
    global packet
    frequency = int(freqSlider.val)
    amplitude = ampSlider.val
    #Set the type to above configuration
    if radio.value_selected == 'Sine':
        wType = 0  
    elif radio.value_selected== 'Square':
        wType = 1
    elif radio.value_selected == 'Sawtooth':
        wType = 2
    else:
        wType = 3

    packet[0] = wType
    packet[1] = float2fix7(amplitude)
    packet[2] = (frequency & 0xFFFF) >> 8
    packet[3] = (frequency & 0xFF)

    spi_writeread(packet, True)

def plot():    #Function to create the base plot, make sure to make global the lines, axes, canvas and any part that you would want to update later
    global line, ax, fig
    global xIncrement
    global xValue
    global ymax
    global peakToPeak
    global txt
    ymax = 4

    global frequency
    frequency = 50
    global amplitude
    amplitude = 0.5

    

    #Set up figures and axes
    fig, ax = plt.subplots()
    txt = plt.figtext(0.5,0.8,'initial')
    ax.set_xlim([0, data_width]) #can this be xmax?
    ax.set_ylim([0, ymax])
    fig.canvas = FigureCanvasTkAgg(fig, master=window)
    fig.canvas.show()
    fig.canvas.get_tk_widget().pack(side=TOP, fill=BOTH, expand=1)
    fig.canvas._tkcanvas.pack(side=TOP, fill=BOTH, expand=1)
    line, = ax.plot([], [])
    plt.subplots_adjust(bottom=0.5)

    
    #Sliders and radios
    axcolor = 'lightgoldenrodyellow'
    axXmax = plt.axes([0.2, 0.4, 0.65, 0.03], axisbg=axcolor)
    axYmax = plt.axes([0.2, 0.35, 0.65, 0.03], axisbg=axcolor)

    global xMaxSlider
    global yMaxSlider
    
    funGenText = plt.text(1.25,3.25,'Function Generation')
    funGenText.set_size(20)

    oscilGenText = plt.text(1.75,19.25,'Oscilliscope')
    oscilGenText.set_size(20)
    
    xMaxSlider = Slider(axXmax, 'Zoom:', 0, 10, valinit=0)
    yMaxSlider = Slider(axYmax, 'Y Max:', .1, 5, valinit=ymax)
    xMaxSlider.on_changed(updateAxes)
    yMaxSlider.on_changed(updateAxes)

    axYmax.text(-.5, -2, 'Function Generation:', color='black')

    rax = plt.axes([0.15, .1, 0.15, 0.15], axisbg=axcolor)
    global radio
    radio = RadioButtons(rax, ('Sine', 'Square', 'Sawtooth'))

    global saveButton
    rax2 = plt.axes([0, .1, 0.15, 0.15],axisbg=axcolor)
    saveButton = RadioButtons(rax2, ('No Save', 'Save'))

    freq = plt.axes([.4, 0.2, 0.4, 0.03], axisbg=axcolor)
    amp = plt.axes([.4, 0.15, 0.4, 0.03], axisbg=axcolor)
    
    global freqSlider
    global ampSlider
    freqSlider = Slider(freq, 'Freq:', 1, 750, valinit=frequency)
    ampSlider = Slider(amp, 'Amp:', 0, 1, valinit=amplitude)
    

    freqSlider.on_changed(fcn_gen)
    ampSlider.on_changed(fcn_gen)
    radio.on_clicked(fcn_gen)
    saveButton.on_clicked(save_file)

def updateplot(q1,q2):
    try:
        global window
        global fig
    	global xVal
    	global yVal
    	global tValue
    	global txt
    	
        avgWindowSize = 5

	if (q1.qsize() >= numSamples):
            index = 0
            lastOffsetIndex = 0
            minVolt = sys.maxint
            maxVolt = 0
    	    while ((q1.empty() == False) and (q2.empty() == False) and (index<=numSamples)):
		    x = q2.get()
    		    y = q1.get()
    		    if (y > maxVolt):
                        maxVolt = y

                    if (y < minVolt):
                        minVolt = y
                        
    		    index = index +1
		    xVal.append(x)
    		    yVal.append(y)

    	    peakToPeak = maxVolt - minVolt
            startAvgIndex = int(avgWindowSize/2)
            for t in range(startAvgIndex, len(xVal)-2):
                avgSum = 0
                for v in range(-int(avgWindowSize/2), int(avgWindowSize/2) + 1):
                    avgSum = avgSum + xVal[t + v] 
                xVal[t] = avgSum/avgWindowSize
            
            sampleDropIndex = len(xVal)-1-tValue*samplesToDropMultiple
            
            ax.set_xlim([xVal[0],xVal[sampleDropIndex]])
            ax.get_xaxis().set_visible(False)
            
            line.set_xdata(xVal[:sampleDropIndex])
            line.set_ydata(yVal[:sampleDropIndex])
            txt.set_text('Peak to Peak:'+ str(peakToPeak))

            xVal = []
            yVal = []
            fig.canvas.draw()
            window.after(30,updateplot,q1, q2)

    except:
        print "empty"
        window.after(60,updateplot,q1, q2)


def spi_writeread(data,packetChange):
    global changedPacket
    #figure out how many bytes you are going to receive
    spi.cshigh = False

    if packetChange:
        #data must be an array of 4 bytes (each entry not greater than 255)
        spi.xfer(data)
        
    
    resp = spi.readbytes(4)
    
    voltage = fixToFloat(resp[3], resp[2], resp[1], resp[0])
    #receive the bytes
    spi.cshigh = True
    return voltage

def fixToFloat(llsb, lsb, msb, mmsb):
    number = mmsb << 24
    number |= msb << 16
    number |= lsb << 8
    number |= llsb

    return float(number)/65536.0

# Packet: | 2 Bits Type | 4 Bits Amplitude | 10 Bits Frequency |
# Type: 0 is Sine, 1 is Square, 2 is Sawtooth
def spi_write(data):
    #send the actual data packet
    spi.cshigh = False
    spi.xfer(data)     
    spi.cshigh = True


def simulation(q1,q2):
    while True:    
        #here send any data you want to send to the other process, 
        #can be any pickable object
        global acc
        global xIncrement
        global packet
        global prevPacket
        global changedPacket

        #curTime = int(round(time.time() *1000))
        changedPacket = prevPacket != packet
        packetChange = prevPacket != packet
        y = spi_writeread(packet, packetChange)
        #time.sleep(xIncrement)

        if q1.qsize() <= numSamples:
            acc += xIncrement
            q1.put(y)
            q2.put(acc)
            

        prevPacket = packet


def save_file(x):
    global saveIndex
    if saveButton.value_selected == 'Save':
        path = 'plot'+str(saveIndex)+'.png'
        fig.savefig(path)
        saveIndex += 1
    else:
        print 'Nothing to save'

	
if __name__ == '__main__':
    main()
